"""This Module allows for the listing of users passwords"""
from libpkpass.util import dictionary_filter
from libpkpass.commands.command import Command
from libpkpass.passworddb import PasswordDB
from libpkpass.errors import CliArgumentError

    ####################################################################
class List(Command):
    """This class implements the cli list"""
    ####################################################################
    name = 'list'
    description = 'List passwords you have access to'
    selected_args = Command.selected_args + ['pwstore', 'stdin', 'recovery', 'filter']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################

        passworddb = PasswordDB()
        passworddb.load_from_directory(self.args['pwstore'])

        result = {}
        for pwname, passwordentry in passworddb.pwdb.items():
            if self.args['recovery'] and passwordentry.escrow:
                for rec_list in passwordentry.escrow.keys():
                    recipients = passwordentry.escrow[rec_list]['recipients']
                    for key, value in recipients.items():
                        if key == self.args['identity']:
                            result[pwname] = {
                                'name': passwordentry.metadata['name'],
                                'group': rec_list,
                                'stake_holders': list(recipients.keys()),
                                'distributor': value['distributor'],
                                'minimum_shares': passwordentry.escrow[rec_list]['metadata']['minimum_escrow']
                            }
            elif not self.args['recovery'] and self.args['identity'] in passwordentry.recipients.keys():
                result[pwname] = {
                    'name': passwordentry.metadata['name'],
                    'distributor': passwordentry.recipients[self.args['identity']]['distributor']
                }

        if 'filter' in self.args and self.args['filter']:
            result = dictionary_filter(self.args['filter'], result)

        print("Passwords for '%s':" % self.args['identity'])
        for key, value in result.items():
            print("%s\n  %s\n  %s" %
                  (
                      self.color_print(key + ":",
                                       "first_level"),
                      self.color_print("Distributor: ",
                                       "second_level") + value['distributor'],
                      self.color_print("Name: ",
                                       "second_level") + value['name']
                  ))
        # print("\n%s" % yaml.dump(result, default_flow_style=False))

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



"""This module allows for the creation of passwords"""
import sys
from libpkpass.commands.command import Command
from libpkpass.errors import CliArgumentError, NotThePasswordOwnerError

class Delete(Command):
    """This class implements the CLI functionality of deletion of passwords"""
    name = 'delete'
    description = 'Delete a password in the repository'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'overwrite', 'stdin', 'keypath', 'card_slot']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        safe, owner = self.safety_check()
        if safe or self.args['overwrite']:
            self._confirmation()
        else:
            raise NotThePasswordOwnerError(self.args['identity'], owner, self.args['pwname'])

        ####################################################################
    def _confirmation(self):
        """Verify password to modify"""
        ####################################################################
        yes = {'yes', 'y', 'ye', ''}
        deny = {'no', 'n'}
        confirmation = input("%s: \nDelete this password?(Defaults yes):"
                             % (self.args['pwname']))
        if confirmation.lower() in yes:
            self.delete_pass()
        elif confirmation.lower() in deny:
            sys.exit()
        else:
            print("please respond with yes or no")
            self._confirmation()

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['pwname', 'keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



"""This Module allows for the import of passwords"""
import getpass
import os.path
import yaml
import libpkpass.crypto as crypto
from libpkpass.commands.command import Command
from libpkpass.password import PasswordEntry
from libpkpass.errors import CliArgumentError, LegacyImportFormatError, FileOpenError

    ####################################################################
class Import(Command):
    """This Class implements the cli functionality of import"""
    ####################################################################
    name = 'import'
    description = 'Import passwords that you have saved to a file'
    selected_args = Command.selected_args + ['pwfile', 'stdin', 'nopassphrase', 'dstpwstore',
                                             'card_slot', 'nocrypto',]

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        try:
            contents = ""
            with open(self.args['pwfile'], 'r') as fcontents:
                contents = fcontents.read().strip()
            if self.args['nocrypto']:
                self._file_handler(contents)
            else:
                passwd = getpass.getpass("Please enter the password for the file: ")
                i = 1
                passwords = contents.split("\n")
                db_len = len(passwords)
                for password in passwords:
                    self._file_handler(crypto.sk_decrypt_string(password, passwd))
                    self.progress_bar(i, db_len)
                    i += 1
            print("")
        except IOError:
            raise FileOpenError(self.args['pwfile'], "No such file or directory")

        ####################################################################
    def _file_handler(self, string):
        """This function handles the contents of a file"""
        ####################################################################
        try:
            self._yaml_file(yaml.safe_load(string))
        except (TypeError, yaml.scanner.ScannerError):
            try:
                self._flat_file(string.strip().split("\n"))
            except TypeError:
                raise LegacyImportFormatError

        ####################################################################
    def _flat_file(self, passwords):
        """This function handles the simple key:value pair"""
        ####################################################################
        print("INFO: Flat password file detected, using 'imported' as description \
you can manually change the description in the file if you would like")
        db_len = len(passwords)
        i = 0
        for password in passwords:
            psplit = password.split(":")
            fname = psplit[0].strip()
            pvalue = psplit[1].strip()
            self.args['pwname'] = fname
            self.create_or_update_pass(pvalue, "imported", self.args['identity'])
            self.progress_bar(i, db_len)
            i += 1
        print("")

        ####################################################################
    def _yaml_file(self, password):
        """This function handles the yaml format of pkpass"""
        ####################################################################
        myidentity = self.identities.iddb[self.args['identity']]
        uid = myidentity['uid']
        pwstore = self.args['pwstore']

        self.args['pwname'] = password['metadata']['name']
        plaintext_str = password['recipients'][uid]['encrypted_secret']
        full_path = os.path.join(pwstore, password['metadata']['name'])
        self.args['overwrite'] = True
        plist = list(password['recipients'])
        if os.path.isfile(full_path):
            existing_password = PasswordEntry()
            existing_password.read_password_data(full_path)
            self.args['overwrite'] = False
            password['metadata'] = existing_password['metadata']
            plist += list(existing_password['recipients'])

        description = password['metadata']['description']
        authorizer = password['metadata']['authorizer']
        self.create_or_update_pass(plaintext_str, description, authorizer, plist)

        ####################################################################
    def _validate_args(self):
        """ Ensure arguments are appropriate for this command           """
        ####################################################################
        for argument in ['pwfile', 'keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



#!/usr/bin/env python
"""This module tests the info module"""
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.info as info
from libpkpass.errors import CliArgumentError
from .basetest.basetest import CONFIG, captured_output

class InfoTests(unittest.TestCase):
    """This class tests the info class"""
    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='info', identity='r1',
                                                nopassphrase="true",
                                                pwname='test',
                                                color=False,
                                                config=CONFIG))
    def test_info(self, subparser_name):
        """test decryption functionality"""
        ret = True
        with captured_output() as (out, _):
            info.Info(cli.Cli())
        output = "".join(out.getvalue().strip().split()).split("timestamp")[0]
        check_dict = {
            'Metadata': {
                'Authorizer': 'y',
                'Creator': 'r1',
                'Description': 'y',
                'Name': 'test',
                'Schemaversion': 'v1',
                'Signature': 'None',
                'Recipients': 'r1',
                'TotalRecipients': '1',
            }
        }
        for key, value in check_dict['Metadata'].items():
            if key not in output and value not in output:
                ret = False
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='info', identity='r1',
                                                nopassphrase="true",
                                                pwname=None,
                                                config=CONFIG))
    def test_info_no_pass(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            info.Info(cli.Cli())
        except CliArgumentError as err:
            if err.msg == "'pwname' is a required argument":
                ret = True
        self.assertTrue(ret)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""This module tests the modify module"""
import builtins
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.modify as modify
from libpkpass.errors import CliArgumentError, DecryptionError
from .basetest.basetest import CONFIG

class ModifyTests(unittest.TestCase):
    """This class tests the modify class"""
    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='modify', identity='r3',
                                                nopassphrase="true",
                                                pwname='gentest',
                                                config=CONFIG))
    def test_modify_success(self, subparser_name):
        """test decryption functionality"""
        ret = ""
        try:
            with mock.patch.object(builtins, 'input', lambda _: 'y'):
                modify.Modify(cli.Cli())
        except DecryptionError as error:
            ret = error.msg
        self.assertEqual(ret, "")




#!/usr/bin/env python
"""Boilerplate test Module"""
import unittest

class TestBasicFunction(unittest.TestCase):
    """Boilerplate test class"""
    # def setUp(self):
    #    self.func = BasicFunction()

    def test_1(self):
        """Boilerplate test case"""
        self.assertTrue(True)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""General Utility file for common functionality"""
import sys
import re
import fnmatch
import argparse
import libpkpass
from colored import fg, attr

    ####################################################################
def color_prepare(string, color_type, colorize, theme_map=None):
    """Handle the color output of a given string"""
    ####################################################################
    if theme_map is None:
        theme_map = {}
    color_defaults = {
        "info": "cyan",
        "warning": "yellow",
        "debug": "red",
        "first_level": "magenta",
        "second_level": "green",
        "third_level": "blue"
    }
    color = theme_map[color_type].lower() if (color_type in theme_map) else color_defaults[color_type]
    try:
        return "%s%s%s" % (fg(color), string, attr('reset')) if colorize else string
    except KeyError:
        return "%s%s%s" % (fg(color_defaults[color_type]), string, attr('reset')) if colorize else string


    ####################################################################
def set_default_subparser(self, name, args=None, positional_args=0):
    """Set default subparser to interpreter"""
    ####################################################################
    subparser_found = False
    for arg in sys.argv[1:]:
        if arg in ['-h', '--help']:
            break
    else:
        for action in self._subparsers._actions:
            if not isinstance(action, argparse._SubParsersAction):
                continue
            for _ in sys.argv[1:]:
                subparser_found = True
        if not subparser_found:
            if args is None:
                sys.argv.insert(len(sys.argv) - positional_args, name)
            else:
                args.insert(len(args) - positional_args, name)

    ####################################################################
def show_version():
    """return the version number in the VERSION file"""
    ####################################################################
    return libpkpass.__version__

    ####################################################################
def sort(lst):
    """Sort our alphanumeric keys"""
    ####################################################################
    lst = [str(i) for i in lst]
    lst.sort()
    return [int(i) if i.isdigit() else i for i in lst]

    ####################################################################
def dictionary_filter(string_match, dictionary, secondary_check=None):
    """Filter out our dictionary"""
    ####################################################################
    key_list = []
    key_list = fnmatch.filter(dictionary.keys(), string_match)

    # If not a fileglob match, try regex
    if not key_list:
        try:
            key_list = [k for k in dictionary.keys() if re.search(string_match, k)]
        except re.error as err:
            print(err)

    if not secondary_check:
        return {k: dictionary[k] for k in key_list}
    return {k: dictionary[k] for k in key_list
            if secondary_check[0] in dictionary[k][secondary_check[1]].keys()}



#!/usr/bin/env python
"""This module tests the rename module"""
import builtins
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.rename as rename
from libpkpass.errors import DecryptionError, CliArgumentError
from .basetest.basetest import CONFIG, BADPIN

class RenameTests(unittest.TestCase):
    """This class tests the rename class"""

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='rename', identity='bleh',
                                                nopassphrase="true",
                                                all=None,
                                                pwname='test',
                                                rename='retest',
                                                config=CONFIG))
    def test_recipient_not_in_database(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            rename.Rename(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "Error: Your user 'bleh' is not in the recipient database":
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='rename', identity='r1',
                                                nopassphrase="true",
                                                all=True,
                                                pwname='test',
                                                rename='retest',
                                                overwrite="true",
                                                config=CONFIG))
    def test_rename_success(self, subparser_name):
        """test decryption functionality"""
        ret = True
        try:
            with mock.patch.object(builtins, 'input', lambda _: 'y'):
                rename.Rename(cli.Cli())
            with mock.patch('argparse.ArgumentParser.parse_args',
                            return_value=argparse.Namespace(
                                subparser_name='rename', identity='r1',
                                nopassphrase="true",
                                all=True,
                                pwname='retest',
                                rename='test',
                                overwrite="true",
                                config=CONFIG)):
                with mock.patch.object(builtins, 'input', lambda _: 'y'):
                    rename.Rename(cli.Cli())
        except DecryptionError as error:
            if error.msg == BADPIN:
                ret = False
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='rename', identity='r1',
                                                nopassphrase="true",
                                                all=True,
                                                pwname='test',
                                                overwrite="true",
                                                config=CONFIG))
    def test_rename_cli_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            rename.Rename(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "'rename' is a required argument":
                ret = True
        self.assertTrue(ret)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""This Module handles the crypto functions i.e. encryption and decryption"""
import base64
import tempfile
import os
import hashlib
import shutil
from subprocess import Popen, PIPE, STDOUT, DEVNULL
from pem import parse_file
from cryptography.fernet import Fernet, InvalidToken
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from libpkpass.util import color_prepare
from libpkpass.errors import EncryptionError, DecryptionError, SignatureCreationError, X509CertificateError

    ##############################################################################
def handle_python_strings(string):
    """handles py2/3 incompatiblities"""
    ##############################################################################
    if not isinstance(string, bytes):
        string = string.encode("ASCII")
    return string

    ##############################################################################
def pk_encrypt_string(plaintext_string, certificate):
    """ Encrypt and return a base 64 encoded string for the provided identity"""
    ##############################################################################

    plaintext_derived_key = Fernet.generate_key()
    if isinstance(certificate, bytes):
        with tempfile.NamedTemporaryFile(delete=False) as fname:
            fname.write(certificate)
            cert_file_path = fname.name
    else:
        cert_file_path = certificate['certificate_path']
    command = ['openssl', 'rsautl', '-inkey', cert_file_path, '-certin', '-encrypt', '-pkcs']
    proc = Popen(command, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
    stdout, _ = proc.communicate(input=plaintext_derived_key)
    if isinstance(certificate, bytes):
        os.unlink(fname.name)

    if proc.returncode != 0:
        raise EncryptionError("Error encrypting derived key: %s" % stdout)
    ciphertext_derived_key = stdout

    fern = Fernet(plaintext_derived_key)
    encrypted_string = fern.encrypt(handle_python_strings(plaintext_string))

    return (encrypted_string, base64.urlsafe_b64encode(handle_python_strings(ciphertext_derived_key)))

    ##############################################################################
def print_card_info(card_slot, identity, verbosity, color, theme_map):
    """Inform the user what card is selected"""
    ##############################################################################
    if 'key_path' not in identity:
        command = ['pkcs11-tool', '-L']
        proc = Popen(command, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
        stdout, _ = proc.communicate()
        out_list = handle_python_strings(stdout).split(b'Slot')
        if verbosity > 1:
            print_all_slots(stdout, color, theme_map)
        for out in out_list[1:]:
            stripped = out.decode("ASCII").strip()
            if int(stripped[0]) == int(card_slot):
                verbosity = verbosity + 1 if verbosity < 2 else 2
                stripped = ("\n").join(stripped.split('\n')[:verbosity])
                stripped = "Using Slot" + stripped
                print("%s" % (color_prepare(stripped, "info", color, theme_map)))

    ##############################################################################
def print_all_slots(slot_info, color, theme_map):
    """Print all slots and cards available"""
    ##############################################################################
    columns = int(shutil.get_terminal_size().columns) // 4
    print(color_prepare("#" * columns, "debug", color, theme_map))
    print(handle_python_strings(slot_info).decode("ASCII").strip())
    print(color_prepare("#" * columns, "debug", color, theme_map))

    ##############################################################################
def pk_decrypt_string(ciphertext_string, ciphertext_derived_key, identity, passphrase, card_slot=None):
    """ Decrypt a base64 encoded string for the provided identity"""
    ##############################################################################
    ciphertext_derived_key = handle_python_strings(ciphertext_derived_key)
    if 'key_path' in identity:
        command = ['openssl', 'rsautl', '-inkey', identity['key_path'], '-decrypt', '-pkcs']
        proc = Popen(command, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
        stdout, _ = proc.communicate(
            input=base64.urlsafe_b64decode(ciphertext_derived_key))
        plaintext_derived_key = stdout
    else:
        # We've got to use pkcs15-crypt for PIV cards, and it only supports pin on
        # command line (insecure) or via stdin.  So, we have to put ciphertext into
        # a file for pkcs15-crypt to read.  YUCK!
        with tempfile.NamedTemporaryFile(delete=False) as fname:
            fname.write(base64.urlsafe_b64decode(ciphertext_derived_key))
        command = ['pkcs15-crypt', '--decipher', '--raw', '--pkcs', '--input', fname.name]
        if card_slot is not None:
            command.extend(['--reader', str(card_slot)])
        command.extend(['--pin', '-'])
        proc = Popen(command, stdout=PIPE, stdin=PIPE, stderr=DEVNULL)
        stdout, _ = proc.communicate(input=passphrase.encode('ASCII'))
        os.unlink(fname.name)
        try:
            plaintext_derived_key = stdout
        except IndexError:
            raise DecryptionError(stdout)

    if proc.returncode != 0:
        raise DecryptionError(stdout)

    fern = Fernet(plaintext_derived_key)

    plaintext_string = fern.decrypt(handle_python_strings(ciphertext_string))
    return plaintext_string.decode("ASCII")

    ##############################################################################
def pk_sign_string(string, identity, passphrase, card_slot=None):
    """ Compute the hash of string and create a digital signature """
    ##############################################################################
    stringhash = hashlib.sha256(string.encode("ASCII")).hexdigest()
    if 'key_path' in identity:
        command = ['openssl', 'rsautl', '-sign', '-inkey', identity['key_path']]
        proc = Popen(command, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
        stdout, _ = proc.communicate(input=stringhash.encode('ASCII'))
        signature = base64.urlsafe_b64encode(handle_python_strings(stdout))
    else:
        # We've got to use pkcs15-crypt for PIV cards, and it only supports pin on
        # command line (insecure) or via stdin.  So, we have to put signature text
        # into a file for pkcs15-crypt to read.  YUCK!
        with tempfile.NamedTemporaryFile(delete=False) as fname:
            fname.write(stringhash.encode('ASCII'))
        out = tempfile.NamedTemporaryFile(delete=False)
        command = ['pkcs15-crypt', '--sign', '-i', fname.name, '-o', out.name, '--pkcs1', '--raw']
        if card_slot is not None:
            command.extend(['--reader', str(card_slot)])
        command.extend(['--pin', '-'])
        proc = Popen(command, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
        stdout, _ = proc.communicate(input=passphrase.encode('ASCII'))
        out.close()

        with open(out.name, 'rb') as sigfile:
            signature = base64.urlsafe_b64encode(handle_python_strings(sigfile.read()))

        os.unlink(fname.name)
        os.unlink(out.name)

    if proc.returncode != 0:
        raise SignatureCreationError(stdout)

    return signature

    ##############################################################################
def pk_verify_signature(string, signature, identity):
    """ Compute the hash of string and verify the digital signature """
    ##############################################################################
    stringhash = hashlib.sha256(string.encode("ASCII")).hexdigest()
    for cert in parse_file(identity['certificate_path']):
        with tempfile.NamedTemporaryFile(delete=False) as fname:
            fname.write(cert.as_bytes())
        command = ['openssl', 'rsautl', '-inkey', fname.name, '-certin', '-verify']
        proc = Popen(command, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
        stdout, _ = proc.communicate(input=base64.urlsafe_b64decode(handle_python_strings(signature)))
        os.unlink(fname.name)
        if stdout.decode("ASCII") == stringhash:
            return True

    return stdout.decode("ASCII") == stringhash

    ##############################################################################
def pk_verify_chain(certificate, cabundle):
    """ Verify the publickey trust chain against a CA Bundle and return True if valid"""
    ##############################################################################
    command = ['openssl', 'verify', '-CAfile', cabundle]
    proc = Popen(command, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
    stdout, _ = proc.communicate(input=certificate)

    return stdout.decode("ASCII").rstrip() == "stdin: OK"

    ##############################################################################
def get_cert_fingerprint(cert):
    """ Return the modulus of the x509 certificate of the identity """
    ##############################################################################
    # SHA1 Fingerprint=F9:9D:71:54:55:BE:99:24:6A:5E:E0:BB:48:F9:63:AE:A2:05:54:98
    return get_cert_element(cert, 'fingerprint').split('=')[1]

    ##############################################################################
def get_cert_subject(cert):
    """ Return the subject DN of the x509 certificate of the identity """
    ##############################################################################
    # subject= /C=US/O=Entrust/OU=Certification Authorities/OU=Entrust Managed Services SSP CA
    return ' '.join(get_cert_element(cert, 'subject').split(' ')[1:])

    ##############################################################################
def get_cert_issuer(cert):
    """ Return the issuer DN of the x509 certificate of the identity """
    ##############################################################################
    # issuer= /C=US/O=Entrust/OU=Certification Authorities/OU=Entrust Managed Services SSP CA
    return ' '.join(get_cert_element(cert, 'issuer').split(' ')[1:])

    ##############################################################################
def get_cert_enddate(cert):
    """ Return the issuer DN of the x509 certificate of the identity """
    ##############################################################################
    return get_cert_element(cert, 'enddate').split('=')[1]

    ##############################################################################
def get_cert_issuerhash(cert):
    """ Return the issuer DN of the x509 certificate of the identity """
    ##############################################################################
    return get_cert_element(cert, 'issuer_hash')

    ##############################################################################
def get_cert_subjecthash(cert):
    """ Return the issuer DN of the x509 certificate of the identity """
    ##############################################################################
    return get_cert_element(cert, 'subject_hash')

    ##############################################################################
def get_cert_element(cert, element):
    """ Return an arbitrary element of an x509 certificate """
    ##############################################################################
    command = ['openssl', 'x509', '-noout', ("-%s" % element)]
    proc = Popen(command, stdout=PIPE, stdin=PIPE, stderr=STDOUT)
    stdout, _ = proc.communicate(input=cert)
    if proc.returncode != 0:
        raise X509CertificateError(stdout)

    try:
        return stdout.decode("ASCII").rstrip()
    except IndexError:
        raise X509CertificateError(stdout)

    ##############################################################################
def sk_encrypt_string(plaintext_string, key):
    """ Symmetrically Encrypt and return a base 64 encoded string using the provided secret"""
    ##############################################################################
    fern = Fernet(hash_password(key))
    encrypted_string = fern.encrypt(plaintext_string.encode('ASCII'))
    return base64.urlsafe_b64encode(handle_python_strings(encrypted_string))

    ##############################################################################
def sk_decrypt_string(ciphertext_string, key):
    """ Symmetrically Decrypt a base64 encoded string using the provided key"""
    ##############################################################################
    fern = Fernet(hash_password(key))
    ciphertext_string = handle_python_strings(ciphertext_string)
    try:
        plaintext_string = fern.decrypt(base64.urlsafe_b64decode(handle_python_strings(ciphertext_string)))
        return plaintext_string.decode("ASCII")
    except InvalidToken:
        # handle some legacy stuff from NCCS
        try:
            plaintext_string = fern.decrypt(ciphertext_string)
            return plaintext_string.decode("ASCII")
        except InvalidToken:
            raise DecryptionError("Incorrect Password")

    ##############################################################################
def hash_password(password):
    """Hash people's password"""
    ##############################################################################
    password = handle_python_strings("%s" % password)
    kdf = PBKDF2HMAC(algorithm=hashes.SHA256(), length=32, salt=b"salt",
                     iterations=100000, backend=default_backend())
    return base64.urlsafe_b64encode(handle_python_strings(kdf.derive(password)))



"""This Module allows for the listing of recipients"""
import libpkpass.util as util
from libpkpass.commands.command import Command
from libpkpass.errors import CliArgumentError

    ####################################################################
class Listrecipients(Command):
    """This class implements the cli functionality to list recipients"""
    ####################################################################
    name = 'listrecipients'
    description = 'List the recipients that pkpass knows about'
    selected_args = Command.selected_args + ['stdin', 'filter']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                  """
        ####################################################################

        if self.args['verbosity'] != -1:
            print('Certificate store: "%s"' % self.args['certpath'])
            print('Key store: "%s"' % self.args['keypath'])
            print('CA Bundle file: "%s"' % self.args['cabundle'])
            print('Looking for Key Extension: "%s"' %
                  self.identities.extensions['key'])
            print('Looking for Certificate Extension: "%s"' %
                  self.identities.extensions['certificate'])

            print("Loaded %s identities:\n" % len(self.identities.iddb.keys()))

        # we don't want to overwrite iddb because of the interpreter
        if 'filter' in self.args and self.args['filter']:
            tempiddb = util.dictionary_filter(self.args['filter'], self.identities.iddb)
        else:
            tempiddb = self.identities.iddb
        for key, _ in tempiddb.items():
            self._print_identity(key)

        ####################################################################
    def _print_identity(self, key):
        """Print off identity"""
        ####################################################################
        print("%s" % self.color_print(self.identities.iddb[key]['uid'] + ":", "first_level"))
        print("\t%s" % self.color_print("certs:", "second_level"))
        for cert in self.identities.iddb[key]['certs']:
            for info in ['verified', 'subject', 'subjecthash', 'issuer', 'issuerhash', 'fingerprint', 'enddate']:
                print("\t\t%s %s" %
                      (self.color_print(info + ":", "third_level"),
                       cert[info]))
            print()

        ####################################################################
    def _validate_args(self):
        """ Ensure arguments are appropriate for this command       """
        ####################################################################
        for argument in ['keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)

        ####################################################################
    def _validate_identities(self, _=None):
        """ Ensure identities are appropriate for this command       """
        ####################################################################
        return



#!/usr/bin/env python
"""This module tests the listrecipients module"""
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.listrecipients as listrecipients
from .basetest.basetest import CONFIG, captured_output

class ListrecipientsTests(unittest.TestCase):
    """This class tests the listrecipients class"""
    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='listrecipients', identity='r1',
                                                nopassphrase="true",
                                                pwname='test',
                                                color=False,
                                                filter="r3",
                                                config=CONFIG))
    def test_listrecipients(self, subparser_name):
        """test decryption functionality"""
        with captured_output() as (out, _):
            listrecipients.Listrecipients(cli.Cli())
        output = "".join(out.getvalue().strip().split()).split("subject")[0]
        testout = 'Certificatestore:"test/pki/intermediate/certs"Keystore:"test/pki/intermediate/private"CABundlefile:"test/pki/intermediate/certs/ca-bundle"LookingforKeyExtension:".key"LookingforCertificateExtension:"[\'.cert\',\'.crt\']"Loaded4identities:r3:certs:verified:True'
        self.assertEqual(output, testout)

if __name__ == '__main__':
    unittest.main()



"""This Module handles the identitydb object"""
import os
import tempfile
from threading import Thread
from pem import parse_file
import libpkpass.crypto as crypto
from libpkpass.errors import FileOpenError, CliArgumentError

    ##########################################################################
class IdentityDB():
    """ User database class.  Contains information about the identities of and
        things pertinent to recipients and their groups and keys.              """
    ##########################################################################

    def __init__(self):
        self.extensions = {'certificate': ['.cert', '.crt'],
                           'key': '.key'}
        self.cabundle = ""
        self.iddb = {}

    def __repr__(self):
        return "%s(%r)" % (self.__class__, self.__dict__)

    def __str__(self):
        return "%r" % self.__dict__

    def _load_certs_from_external(self, connection_map, nocache):
        if 'base_directory' in connection_map and connection_map['base_directory']:
            temp_dir = connection_map['base_directory']
            del connection_map['base_directory']
        else:
            temp_dir = str(tempfile.gettempdir())

        for key, value in connection_map.items():
            dirname = os.path.join(temp_dir, str(key))
            if not os.path.exists(dirname):
                os.makedirs(dirname)
            if nocache or not os.listdir(dirname):
                encoded = key
                connector = encoded.lower()
                connector = __import__(connector, fromlist=[encoded])
                connector = getattr(connector, encoded)
                connector = connector(value)
                certs = connector.list_certificates()
                for name, certlist in certs.items():
                    with open(os.path.join(dirname, str(name)) +  str(self.extensions['certificate'][0]), 'w') as tmpcert:
                        tmpcert.write("\n".join(certlist))

            self._load_from_directory(dirname, 'certificate')

        #######################################################################
    def _load_from_directory(self, path, filetype):
        """ Helper function to read in (keys|certs) and store them correctly """
        #######################################################################
        try:
            for fname in os.listdir(path):
                if fname.endswith(tuple(self.extensions[filetype])):
                    uid = fname.split('.')[0]
                    filepath = os.path.join(path, fname)
                    try:
                        self.iddb[uid]["%s_path" % filetype] = filepath
                    except KeyError as error:
                        identity = {'uid': fname.split('.')[0],
                                    "%s_path" % filetype: filepath}
                        self.iddb[identity['uid']] = identity
        except OSError as error:
            raise FileOpenError(path, str(error.strerror))

        #######################################################################
    def load_certs_from_directory(self,
                                  certpath,
                                  verify_on_load=False,
                                  connectmap=None,
                                  nocache=False):
        """ Read in all x509 certificates from directory and name them as found """
        #######################################################################
        if connectmap:
            self._load_certs_from_external(connectmap, nocache)
        if certpath:
            self._load_from_directory(certpath, 'certificate')
        if verify_on_load:
            threads = []
            for identity, _ in self.iddb.items():
                threads.append(Thread(target=self.verify_identity,
                                      args=(identity, [])))
                threads[-1].start()
            for thread in threads:
                thread.join()

        #######################################################################
    def verify_identity(self, identity, results):
        """ Read in all rsa keys from directory and name them as found
        results is a meaningless parameter, but is required to make threading work
        """
        #######################################################################
        try:
            self.iddb[identity]['cabundle'] = self.cabundle
            self.iddb[identity]['certs'] = []
            for cert in parse_file(self.iddb[identity]['certificate_path']):
                cert = cert.as_bytes()
                cert_dict = {}
                cert_dict['cert_bytes'] = cert
                cert_dict['verified'] = crypto.pk_verify_chain(cert, self.iddb[identity]['cabundle'])
                cert_dict['fingerprint'] = crypto.get_cert_fingerprint(cert)
                cert_dict['subject'] = crypto.get_cert_subject(cert)
                cert_dict['issuer'] = crypto.get_cert_issuer(cert)
                cert_dict['enddate'] = crypto.get_cert_enddate(cert)
                cert_dict['issuerhash'] = crypto.get_cert_issuerhash(cert)
                cert_dict['subjecthash'] = crypto.get_cert_subjecthash(cert)
                self.iddb[identity]['certs'].append(cert_dict)
        except KeyError:
            raise CliArgumentError(
                "Error: Recipient '%s' is not in the recipient database" % identity)

        #######################################################################
    def load_keys_from_directory(self, path):
        """ Read in all rsa keys from directory and name them as found """
        #######################################################################
        if os.path.isdir(path):
            self._load_from_directory(path, 'key')
        # We used to print a warning on else here, but it seems unnecessary;
        # if a user is using PIVs/Smartcards, this warning could just annoy
        # them needlessly



#!/usr/bin/env python
"""This module tests the list module"""
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.list as pklist
from libpkpass.errors import CliArgumentError
from .basetest.basetest import CONFIG, captured_output

PASSWORD_LIST_0 = "Passwordsfor'r2':"
PASSWORD_LIST_1 = "Passwordsfor'r1':test/passwords/test:Distributor:r1Name:test"

class ListTests(unittest.TestCase):
    """This class tests the list class"""

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='list', identity='bleh',
                                                nopassphrase="true",
                                                config=CONFIG,
                                                color="false"))
    def test_recipient_not_in_database(self, subparser_name):
        """test bad recipient functionality"""
        ret = False
        try:
            pklist.List(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "Error: Your user 'bleh' is not in the recipient database":
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='list', identity='r2',
                                                nopassphrase="true",
                                                config=CONFIG,
                                                color="false"))
    def test_list_none(self, subparser_name):
        """test list functionality"""
        ret = False
        with captured_output() as (out, _):
            pklist.List(cli.Cli())
        output = "".join(out.getvalue().strip().split())
        self.assertEqual(output, PASSWORD_LIST_0)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='list', identity='r1',
                                                nopassphrase="true",
                                                config=CONFIG,
                                                color="false"))
    def test_list_one(self, subparser_name):
        """test list functionality"""
        ret = False
        with captured_output() as (out, _):
            pklist.List(cli.Cli())
        output = "".join(out.getvalue().strip().split())
        self.assertEqual(output, PASSWORD_LIST_1)

if __name__ == '__main__':
    unittest.main()



"""This module allows for the updating of passwords"""
import getpass
import os
import libpkpass.util as util
from libpkpass.password import PasswordEntry
from libpkpass.commands.command import Command
from libpkpass.errors import CliArgumentError, PasswordMismatchError, NotThePasswordOwnerError,\
        BlankPasswordError

    ####################################################################
class Update(Command):
    """This class implements the CLI functionality of updating existing passwords"""
    ####################################################################
    name = 'update'
    description = 'Change a password value and redistribute to recipients'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'overwrite', 'stdin', 'keypath',
                                             'nopassphrase', 'nosign', 'card_slot', 'escrow_users',
                                             'min_escrow', 'noescrow']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        password = PasswordEntry()
        password.read_password_data(os.path.join(
            self.args['pwstore'], self.args['pwname']))
        safe, owner = self.safety_check()
        if safe or self.args['overwrite']:
            self.recipient_list = password['recipients'].keys()
            print("The following list of users are the current distribution list:")
            print(", ".join(util.sort(self.recipient_list)))
            correct_distribution = input("Is this list correct? (y/N) ")
            if not correct_distribution or correct_distribution.lower()[0] == 'n':
                self._new_distribution()

            self._validate_identities(self.recipient_list)

            password1 = getpass.getpass("Enter updated password: ")
            if password1.strip() == "":
                raise BlankPasswordError
            password2 = getpass.getpass("Enter updated password again: ")
            if password1 != password2:
                raise PasswordMismatchError

            # This is due to a poor naming convention; we don't want to go through the
            # create or update, because we are not updating a single record in the yaml
            # we are burning it to the ground and redoing the entire thing
            self.create_pass(password1,
                             password['metadata']['description'],
                             password['metadata']['authorizer'],
                             self.recipient_list)
        else:
            raise NotThePasswordOwnerError(self.args['identity'], owner, self.args['pwname'])

        ####################################################################
    def _new_distribution(self):
        """enter a new recipient list"""
        ####################################################################
        breaker = False
        while not breaker:
            self.recipient_list = input("Please enter a comma delimited list: ")
            self.recipient_list = list(set(self._convert_strings_to_list(self.recipient_list)))
            print(self.recipient_list)
            try:
                self._validate_identities(self.recipient_list)
                breaker = True
            except CliArgumentError as err:
                print(err)
                continue

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['pwname', 'keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



#!/usr/bin/env python
"""This module tests the create module"""
import getpass
import builtins
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.create as create
from libpkpass.errors import CliArgumentError, DecryptionError
from .basetest.basetest import CONFIG

class CreateTests(unittest.TestCase):
    """This class tests the create class"""
    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='create', identity='r1',
                                                nopassphrase="true",
                                                pwname='test',
                                                config=CONFIG))
    def test_create_success(self, subparser_name):
        """test decryption functionality"""
        ret = ""
        try:
            with mock.patch.object(builtins, 'input', lambda _: 'y'):
                with mock.patch.object(getpass, 'getpass', lambda _: 'y'):
                    create.Create(cli.Cli())
        except DecryptionError as error:
            ret = error.msg
        self.assertEqual(ret, "")

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='create', identity='r1',
                                                nopassphrase="true",
                                                pwname=None,
                                                config=CONFIG))
    def test_create_no_pass(self, subparser_name):
        """test decryption functionality"""
        ret = ""
        try:
            with mock.patch.object(builtins, 'input', lambda _: 'y'):
                with mock.patch.object(getpass, 'getpass', lambda _: 'y'):
                    create.Create(cli.Cli())
        except CliArgumentError as error:
            ret = error.msg
        self.assertEqual(ret, "'pwname' is a required argument")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""This module tests the distribute module"""
import builtins
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.distribute as distribute
from libpkpass.errors import CliArgumentError, DecryptionError
from .basetest.basetest import CONFIG

class DistributeTests(unittest.TestCase):
    """This class tests the distribute class"""
    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='distribute', identity='bleh',
                                                nopassphrase="true",
                                                pwname='test',
                                                config=CONFIG))
    def test_recipient_not_in_database(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            distribute.Distribute(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "Error: Your user 'bleh' is not in the recipient database":
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='distribute', identity='r1',
                                                nopassphrase="true",
                                                pwname=None,
                                                config=CONFIG))
    def test_distribute_cli_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            distribute.Distribute(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "'pwname' is a required argument":
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='distribute', identity='r1',
                                                nopassphrase="true",
                                                pwname='test',
                                                config=CONFIG))
    def test_distribute_cli_decrypt(self, subparser_name):
        """test decryption functionality"""
        ret = ""
        try:
            with mock.patch.object(builtins, 'input', lambda _: 'y'):
                distribute.Distribute(cli.Cli())
        except DecryptionError as error:
            ret = error.msg
        self.assertEqual(ret, "")


if __name__ == '__main__':
    unittest.main()



"""This module is used to process the decryption of keys"""
import os
import fnmatch
from libpkpass.commands.command import Command
from libpkpass.password import PasswordEntry
from libpkpass.errors import PasswordIOError, CliArgumentError, NotARecipientError, DecryptionError

    ####################################################################
class Show(Command):
    """This class is used as a command object and parses information passed through
    the CLI to show passwords that have been distributed to users"""
    ####################################################################
    name = 'show'
    description = 'Display a password'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'stdin', 'keypath', 'nopassphrase',
                                             'noverify', 'card_slot', 'all', 'recovery', 'ignore_decrypt']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                  """
        ####################################################################
        password = PasswordEntry()
        myidentity = self.identities.iddb[self.args['identity']]

        if self.args['all']:
            try:
                self._walk_dir(self.args['pwstore'], password, myidentity, self.args['ignore_decrypt'])
            except DecryptionError as err:
                raise err
        elif self.args['pwname'] is None:
            raise PasswordIOError("No password supplied")
        else:
            self._decrypt_wrapper(
                self.args['pwstore'], password, myidentity, self.args['pwname'])

        ####################################################################
    def _walk_dir(self, directory, password, myidentity, ignore_decrypt=False):
        """Walk our directory searching for passwords"""
        ####################################################################
        # os.walk returns root, dirs, and files we just need files
        for root, _, pwnames in os.walk(directory):
            for pwname in pwnames:
                if self.args['pwname'] is None or\
                        fnmatch.fnmatch(os.path.join(root, pwname), self.args['pwname']):
                    try:
                        self._decrypt_wrapper(
                            root, password, myidentity, pwname)
                    except DecryptionError as err:
                        if ignore_decrypt:
                            print(err)
                            continue
                        raise
                    except NotARecipientError:
                        continue

        ####################################################################
    def _handle_escrow_show(self, password, myidentity):
        """This populates the user's escrow as passwords"""
        ####################################################################
        myescrow = []
        if password.escrow:
            for key, value in password['escrow'].items():
                if myidentity['uid'] in value['recipients'].keys():
                    myescrow.append([value['recipients'][myidentity['uid']], key])
        return myescrow

        ####################################################################
    def _decrypt_wrapper(self, directory, password, myidentity, pwname):
        """Decide whether to decrypt normally or for escrow"""
        ####################################################################
        password.read_password_data(os.path.join(directory, pwname))
        myescrow = []
        if self.args['recovery']:
            myescrow = self._handle_escrow_show(password, myidentity)
        if myescrow:
            for share in myescrow:
                password['recipients'][myidentity['uid']] = share[0]
                print("Share for escrow group: %s" % share[1])
                self._decrypt_password_entry(password, myidentity)
        else:
            self._decrypt_password_entry(password, myidentity)

        ####################################################################
    def _decrypt_password_entry(self, password, myidentity):
        """This decrypts a given password entry"""
        ####################################################################
        plaintext_pw = password.decrypt_entry(
            identity=myidentity, passphrase=self.passphrase, card_slot=self.args['card_slot'])
        if not self.args['noverify']:
            result = password.verify_entry(
                myidentity['uid'], self.identities)
            if not result['sigOK']:
                print("warning: could not verify that '%s' correctly signed your password entry." %
                      result['distributor'])
            if not result['certOK']:
                print("Warning: could not verify the certificate authenticity of user '%s'." %
                      result['distributor'])

        print(("%s: %s") % (self.color_print(password.metadata['name'],
                                             "first_level"), plaintext_pw))

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



"""This module allows for inspecting metadata passwords"""
import os
import datetime
from libpkpass.password import PasswordEntry
from libpkpass.commands.command import Command
from libpkpass.errors import CliArgumentError


    ####################################################################
class Info(Command):
    """This class implements the CLI functionality of creation of passwords"""
    ####################################################################
    name = 'info'
    description = 'Create a new password entry and encrypt it for yourself'
    selected_args = Command.selected_args + ['pwname', 'pwstore']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        password = PasswordEntry()
        password.read_password_data(os.path.join(self.args['pwstore'], self.args['pwname']))

        # Metadata
        print(self.color_print("Metadata:", "first_level"))
        for key, value in password.metadata.items():
            print("  %s: %s" % (self.color_print(key.capitalize(), "second_level"),
                                value))

        # Escrow
        if password.escrow:
            print(self.color_print("\nEscrow Groups:", "first_level"))
            for group_key, group_value in password.escrow.items():
                print("  %s:" % self.color_print(group_key, "second_level"))
                for key, value in group_value['metadata'].items():
                    print("    %s %s" % (self.color_print(key + ":", "third_level"), str(value)))

                print("    %s %s" % (self.color_print("Share Holders:", "third_level"),
                                     ', '.join(list(group_value['recipients'].keys()))))
                print("    %s %s" % (self.color_print("Total Group Share Holders:", "third_level"),
                                     len(list(group_value['recipients'].keys()))))

                timestamp_list = [x['timestamp'] for x in list(group_value['recipients'].values()) if 'timestamp' in x]
                if timestamp_list:
                    timestamp = int(round(float(min(timestamp_list))))
                    timestamp = datetime.datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d %H:%M:%S')
                    print("    %s %s" % (self.color_print("Group creation time:", "third_level"),
                                         timestamp))

        # Recipients
        print("%s %s" % (self.color_print("\nRecipients:", "first_level"),
                         ', '.join(list(password.recipients.keys()))))
        print("%s %s" % (self.color_print("Total Recipients:", "first_level"),
                         len(list(password.recipients.keys()))))
        rec_timestamp = int(round(float(min([x['timestamp'] for x in list(password.recipients.values()) if 'timestamp' in x]))))
        rec_timestamp = datetime.datetime.fromtimestamp(rec_timestamp).strftime('%Y-%m-%d %H:%M:%S')
        print("%s %s" % (self.color_print("Earliest distribute timestamp:", "first_level"),
                         rec_timestamp))

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['pwname', 'keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



#!/usr/bin/env python
"""This module tests the update module"""
import getpass
import builtins
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.update as update
from libpkpass.errors import DecryptionError
from .basetest.basetest import CONFIG

class UpdateTests(unittest.TestCase):
    """This class tests the update class"""
    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='update', identity='r3',
                                                nopassphrase="true",
                                                pwname='gentest',
                                                config=CONFIG))
    def test_update_success(self, subparser_name):
        """test decryption functionality"""
        ret = ""
        try:
            with mock.patch.object(builtins, 'input', lambda _: 'y'):
                with mock.patch.object(getpass, 'getpass', lambda _: 'y'):
                    update.Update(cli.Cli())
        except DecryptionError as error:
            ret = error.msg
        self.assertEqual(ret, "")



#!/usr/bin/python
"""This Module validates password objects"""


    #############################################################################
def validate_password(password_obj):
    """ Run through password object and validate everything that needs it """
    ##############################################################################

    # Validate that required fields exist
    # Validate that certificate signatures are valid
    # Validate that password length meets whatever
    # Validate that escrow password is there if needed
    # Validate that things encrypted/decrypted successfully
    return


    #############################################################################
def validate_passwords(password_objs):
    """ Run through password object and validate everything that needs it """
    ##############################################################################

    for password_obj in password_objs:
        validate_password(password_obj)



#!/usr/bin/env python
"""This Module implements a CLI"""
import libpkpass.util as util
import libpkpass.commands.pkinterface as pkinterface
import libpkpass.commands.interpreter as interpreter

    ##############################################################################
class Cli(pkinterface.PkInterface):
    """ Class for parsing command line.  Observes subclasses of Command to Register
    those commands in the actions list.                                        """
    ##############################################################################

        ####################################################################
    def __init__(self):
        """ Intialization function for class. Register all subcommands   """
        ####################################################################
        pkinterface.PkInterface.__init__(self)

        interpreter.Interpreter(self)
        self.parser.add_argument(
            '--version', action='store_true', help='Show the version of PkPass and exit')

        self.parser.set_default_subparser(self.parser, name='interpreter')
        self.parsedargs = self.parser.parse_args()
        if 'version' in self.parsedargs and self.parsedargs.version:
            print(util.show_version())
        else:
            self.actions[self.parsedargs.subparser_name].run(self.parsedargs)



#!/usr/bin/env python
"""This module tests the recover module"""
import unittest
import libpkpass.escrow as escrow

PASSWORD = "boomtown"

class EscrowTests(unittest.TestCase):
    """This class tests the recover class"""
    shares = escrow.pk_split_secret(PASSWORD, ["r1", "r2", "r3"], 2)

    def test_spliting_with_no_min(self):
        shares = escrow.pk_split_secret(PASSWORD, ["r1", "r2", "r3"], None)
        passwd = escrow.pk_recover_secret(shares)
        self.assertEqual(passwd, PASSWORD)

    def test_split_secret_all_shares(self):
        """test recovery with all shares functionality"""
        passwd = escrow.pk_recover_secret(self.shares)
        self.assertEqual(passwd, PASSWORD)

    def test_split_secret_min_shares(self):
        """test recovery with min shares functionality"""
        passwd = escrow.pk_recover_secret(self.shares[0:2])
        self.assertEqual(passwd, PASSWORD)

    def test_split_secret_failure(self):
        """test recovery failure with not enough shares"""
        passwd = escrow.pk_recover_secret(self.shares[0:1])
        self.assertNotEqual(passwd, PASSWORD)



#!/usr/bin/env python
"""This module tests the generate module"""
import getpass
import builtins
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.generate as generate
from libpkpass.errors import CliArgumentError, DecryptionError
from .basetest.basetest import CONFIG

class GenerateTests(unittest.TestCase):
    """This class tests the generate class"""
    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='generate', identity='r3',
                                                nopassphrase="true",
                                                pwname='gentest',
                                                config=CONFIG))
    def test_generate_success(self, subparser_name):
        """test decryption functionality"""
        ret = ""
        try:
            with mock.patch.object(builtins, 'input', lambda _: 'y'):
                with mock.patch.object(getpass, 'getpass', lambda _: 'y'):
                    generate.Generate(cli.Cli())
        except DecryptionError as error:
            ret = error.msg
        self.assertEqual(ret, "")

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='generate', identity='r3',
                                                nopassphrase="true",
                                                pwname=None,
                                                config=CONFIG))
    def test_generate_no_pass(self, subparser_name):
        """test decryption functionality"""
        ret = ""
        try:
            with mock.patch.object(builtins, 'input', lambda _: 'y'):
                with mock.patch.object(getpass, 'getpass', lambda _: 'y'):
                    generate.Generate(cli.Cli())
        except CliArgumentError as error:
            ret = error.msg
        self.assertEqual(ret, "'pwname' is a required argument")


if __name__ == '__main__':
    unittest.main()



"""This module is a generic for all pkpass commands"""
import sys
import getpass
import json
import os
import yaml
import libpkpass.util as util
from libpkpass.commands.arguments import ARGUMENTS as arguments
from libpkpass.password import PasswordEntry
from libpkpass.identities import IdentityDB
from libpkpass.crypto import print_card_info
from libpkpass.errors import NullRecipientError, CliArgumentError, FileOpenError, GroupDefinitionError,\
        PasswordIOError, JsonArgumentError, NotThePasswordOwnerError, ConfigParseError

    ##########################################################################
class Command():
    """ Base class for all commands.  Auotmatically registers with cli subparser
    and provides run execution for itself.                                     """
    ##########################################################################
    name = None
    description = None
    selected_args = ['nocache', 'verbosity', 'quiet', 'identity', 'cabundle',
                     'certpath', 'color', 'theme_map']
    passphrase = None

        ##################################################################
    def __init__(self, cli, iddb=None):
        """ Intialization function for class. Register with argparse   """
        ##################################################################
        self.cli = cli
        #default certpath to none because connect string is allowed
        self.args = {
            'ignore_decrypt': False,
            'identity': getpass.getuser(),
            'cabundle': './certs/ca-bundle',
            'keypath': './private',
            'pwstore': './passwords',
            'time': 10,
            'card_slot': None,
            'certpath': None,
            'escrow_users': None,
            'min_escrow': None,
            'no_cache': False,
            'noverify': None,
            'noescrow': False,
            'overwrite': False,
            'recovery': False,
            'rules': 'default',
            'theme_map': None,
            'color': True,
            'verbosity': 0,
            }
        self.recipient_list = []
        self.escrow_and_recipient_list = []
        self.iddbcached = iddb is not None
        self.identities = iddb if iddb else IdentityDB()
        cli.register(self, self.name, self.description)

        ##################################################################
    def register(self, parser):
        """ Registration function for class. Register with argparse      """
        ##################################################################
        for arg in sorted(self.selected_args):
            parser.add_argument(
                *arguments[arg]['args'],
                **arguments[arg]['kwargs'])

        ##################################################################
    def run(self, parsedargs):
        """ Passes the argparse Namespace object of parsed arguments   """
        ##################################################################
        self._run_command_setup(parsedargs)
        self._run_command_execution()

        ##################################################################
    def _convert_strings_to_list(self, argname):
        """ convert argparsed strings to lists for an argument """
        ##################################################################
        if argname in self.args:
            self.args[argname] = self.args[argname].split(",") if self.args[argname] else []
            self.args[argname] = [arg.strip() for arg in self.args[argname] if arg.strip()]
        elif isinstance(argname, str):
            argname = argname.split(",") if argname else []
            return [arg.strip() for arg in argname if arg.strip()]
        return None

        ##################################################################
    def _handle_boolean_args(self, argname):
        ##################################################################
        if isinstance(self.args[argname], str):
            return self.args[argname].upper() == 'TRUE'
        return self.args[argname]

        ##################################################################
    def _handle_filepath_args(self):
        """Does filepath expansion for config args"""
        ##################################################################
        file_path_args = ['cabundle', 'pwstore', 'certpath', 'keypath']
        for arg in file_path_args:
            if arg in self.args and self.args[arg]:
                self.args[arg] = os.path.expanduser(self.args[arg])
        if 'connect' in self.args and self.args['connect'] and \
                'base_directory' in self.args['connect'] and self.args['connect']['base_directory']:
            self.args['connect']['base_directory'] = os.path.expanduser(self.args['connect']['base_directory'])

        ##################################################################
    def _run_command_setup(self, parsedargs):
        """ Passes the argparse Namespace object of parsed arguments   """
        ##################################################################

        # Build a dict out of the argparse args Namespace object and a dict from any
        # configuration files and merge the two with cli taking priority
        cli_args = vars(parsedargs)

        config_args = self._get_config_args(cli_args['config'], cli_args)
        self.args.update(config_args)

        connectmap = self._parse_json_arguments('connect')
        self._handle_filepath_args()

        fles = ['cabundle', 'pwstore']
        for key, value in cli_args.items():
            if value is not None or key not in self.args:
                self.args[key] = value
            if key in fles and not os.path.exists(self.args[key]):
                raise FileOpenError(self.args[key], "No such file or directory")

        # json args
        self.args['color'] = self._handle_boolean_args('color')
        self._convert_strings_to_list('groups')
        self._convert_strings_to_list('users')
        self._convert_strings_to_list('escrow_users')
        self._validate_combinatorial_args()
        self._validate_args()

        verify_on_load = self.args['subparser_name'] in ['listrecipients', 'import', 'interpreter']

        # Build the list of recipients that this command will act on
        self._build_recipient_list()

        # If there are defined repositories of keys and certificates, load them
        if not self.iddbcached:
            self.identities.cabundle = self.args['cabundle']
            self.identities.load_certs_from_directory(
                self.args['certpath'],
                verify_on_load=verify_on_load,
                connectmap=connectmap,
                nocache=self.args['no_cache']
            )
            self.identities.load_keys_from_directory(self.args['keypath'])
            self._validate_identities()

        if 'pwname' in self.args and self.args['pwname']:
            self._resolve_directory_path()
        self.args['card_slot'] = self.args['card_slot'] if self.args['card_slot'] else 0
        if 'nopassphrase' in self.selected_args and not self.args['nopassphrase']:
            if self.args['verbosity'] != -1:
                print_card_info(self.args['card_slot'],
                                self.identities.iddb[self.args['identity']],
                                self.args['verbosity'],
                                self.args['color'],
                                self.args['theme_map'])
            self.passphrase = getpass.getpass("Enter Pin/Passphrase: ")

        ####################################################################
    def _resolve_directory_path(self):
        """This handles how a user inputs the pwname, this tries to be smart
        good luck everybody else"""
        ####################################################################
        pwd = os.getcwd()
        pwd_pwname = os.path.normpath(os.path.join(pwd, self.args['pwname']))
        if self.args['pwstore'] in pwd_pwname:
            self.args['pwname'] = pwd_pwname.replace(self.args['pwstore'] + os.sep, '')

        ##################################################################
    def safety_check(self):
        """ This provides a sanity check that you are the owner of a password."""
        ##################################################################
        try:
            password = PasswordEntry()
            password.read_password_data(os.path.join(self.args['pwstore'], self.args['pwname']))
            return (self.args['identity'] in password['recipients'].keys(), password['metadata']['creator'])
        except PasswordIOError:
            return (True, None)

        ##################################################################
    def update_pass(self, pass_value):
        """Fully updated a password record"""
        ##################################################################
        pass_entry = PasswordEntry()
        pass_entry.read_password_data(os.path.join(self.args['pwstore'], self.args['pwname']))
        swap_pass = PasswordEntry()
        swap_pass.add_recipients(secret=pass_value,
                                 distributor=self.args['identity'],
                                 recipients=[self.args['identity']],
                                 identitydb=self.identities,
                                 passphrase=self.passphrase,
                                 card_slot=self.args['card_slot'],
                                 pwstore=self.args['pwstore']
                                )
        pass_entry['recipients'][self.args['identity']] = swap_pass['recipients'][self.args['identity']]
        pass_entry.write_password_data(os.path.join(self.args['pwstore'], self.args['pwname']),
                                       overwrite=self.args['overwrite'])

        ##################################################################
    def create_pass(self, password1, description, authorizer, recipient_list=None):
        """ This writes password data to a file."""
        ##################################################################
        password_metadata = {}
        password_metadata['description'] = description
        password_metadata['authorizer'] = authorizer
        password_metadata['creator'] = self.args['identity']
        password_metadata['name'] = self.args['pwname']
        if self.args['noescrow']:
            self.args['min_escrow'] = None
            self.args['escrow_users'] = None
        if recipient_list is None:
            recipient_list = [self.args['identity']]

        password = PasswordEntry(**password_metadata)

        password.add_recipients(secret=password1,
                                distributor=self.args['identity'],
                                recipients=recipient_list,
                                identitydb=self.identities,
                                passphrase=self.passphrase,
                                card_slot=self.args['card_slot'],
                                escrow_users=self.args['escrow_users'],
                                minimum=self.args['min_escrow'],
                                pwstore=self.args['pwstore']
                               )

        password.write_password_data(os.path.join(self.args['pwstore'], self.args['pwname']),
                                     overwrite=self.args['overwrite'])

        ##################################################################
    def create_or_update_pass(self, password1, description, authorizer, recipient_list=None):
        """ This creates new or updates existing passwords """
        ##################################################################
        safe, owner = self.safety_check()
        if safe or self.args['overwrite']:
            if owner and owner != self.args['identity']:
                self.update_pass(password1)
            else:
                self.create_pass(password1, description, authorizer, recipient_list)
        else:
            raise NotThePasswordOwnerError(self.args['identity'], owner, self.args['pwname'])

        ##################################################################
    def delete_pass(self):
        """This deletes a password that the user has created, useful for testing"""
        ##################################################################
        filepath = os.path.join(self.args['pwstore'], self.args['pwname'])
        try:
            os.remove(filepath)
        except OSError:
            raise PasswordIOError("Password '%s' not found" % self.args['pwname'])

        ##################################################################
    def rename_pass(self):
        """This renames a password that the user has created"""
        ##################################################################
        oldpath = os.path.join(self.args['pwstore'], self.args['pwname'])
        newpath = os.path.join(self.args['pwstore'], self.args['rename'])
        try:
            os.rename(oldpath, newpath)
            password = PasswordEntry()
            password.read_password_data(newpath)
            password['metadata']['name'] = self.args['rename']
            password.write_password_data(newpath)

        except OSError:
            raise PasswordIOError("Password '%s' not found" % self.args['pwname'])

        ##################################################################
    def _run_command_execution(self):
        """ Passes the argparse Namespace object of parsed arguments   """
        ##################################################################
        raise NotImplementedError

        ##################################################################
    def _build_recipient_list(self):
        """take groups and users and make a SOA for the recipients"""
        ##################################################################
        try:
            if 'groups' in self.args and self.args['groups']:
                self.recipient_list += self._parse_group_membership()
            if 'users' in self.args and self.args['users']:
                self.recipient_list += self.args['users']
            self.recipient_list = [x.strip() for x in list(set(self.recipient_list))]
            self.escrow_and_recipient_list = self.recipient_list + self.args['escrow_users']
            for user in self.escrow_and_recipient_list:
                if str(user) == '':
                    raise NullRecipientError
        except KeyError:  # If this is a command with no users, don't worry about it
            pass

        ##################################################################
    def _parse_group_membership(self):
        """Concat membership of supplied groups"""
        ##################################################################
        member_list = []
        try:
            for group in self.args['groups']:
                member_list += [user.strip() for user in self.args[group.strip()].split(",") if user.strip()]
            return member_list
        except KeyError as err:
            raise GroupDefinitionError(str(err))

        ##################################################################
    def _get_config_args(self, config, cli_args):
        """Return the configuration from the config file"""
        ##################################################################
        try:
            with open(config, 'r') as fname:
                config_args = yaml.safe_load(fname)
            if config_args is None:
                config_args = {}
            return config_args
        except IOError:
            if cli_args['verbosity'] != -1:
                print("INFO: No .pkpassrc file found")
            return {}
        except yaml.parser.ParserError:
            raise ConfigParseError("Parsing error with config file, please check syntax")

        ##################################################################
    def _validate_args(self):
        ##################################################################
        raise NotImplementedError

        ##################################################################
    def _validate_combinatorial_args(self):
        """ This is a weird function name so: combinatorial in this case
            means that one of the 'combinatorial' arguments are required
            however, not all of them are necessarily required.
            Ex: We need certs, we can get this from certpath or connect
            we do not need both of these arguments but at least one is
            required"""
        ##################################################################
        # we want a multi-dim of lists, this way if more combinations come up
        # that would be required in a 1 or more capacity, we just add
        # a list to this list
        args_list = [['certpath', 'connect'], ['certpath', 'keypath']]
        for arg_set in args_list:
            valid = False
            for arg in arg_set:
                if arg in self.args and self.args[arg] is not None:
                    valid = True
                    break
            if not valid:
                raise CliArgumentError(
                    "'%s' or '%s' is required" % tuple(arg_set))

        ##################################################################
    def _parse_json_arguments(self, argument):
        """ Parses the json.loads arguments as dictionaries to use"""
        ##################################################################
        try:
            if argument in self.args and self.args[argument]:
                if isinstance(self.args[argument], dict):
                    return self.args[argument]
                return json.loads(self.args[argument])
            return None
        except ValueError as err:
            raise JsonArgumentError(argument, err)

        ##################################################################
    def _validate_identities(self, swap_list=None):
        """Ensure identities meet criteria for processing"""
        ##################################################################
        if not swap_list:
            swap_list = self.escrow_and_recipient_list
        for recipient in swap_list:
            self.identities.verify_identity(recipient, [])
            if recipient not in self.identities.iddb.keys():
                raise CliArgumentError(
                    "Error: Recipient '%s' is not in the recipient database" %
                    recipient)

        if self.args['identity'] in self.identities.iddb.keys():
            self.identities.verify_identity(self.args['identity'], [])
        else:
            raise CliArgumentError(
                "Error: Your user '%s' is not in the recipient database" %
                self.args['identity'])

        ##################################################################
    def _print_debug(self):
        ##################################################################
        print(self.recipient_list)
        print(self.identities.iddb.keys())

        ##################################################################
    def color_print(self, string, color_type):
        """Handle the color printing for objects"""
        ##################################################################
        return util.color_prepare(string,
                                  color_type,
                                  self.args['color'],
                                  self.args['theme_map'])

        ##################################################################
    def progress_bar(self, value, endvalue, bar_length=50):
        """Allow a command to print off a progress bar"""
        ##################################################################
        percent = float(value) / endvalue
        arrow = '-' * int(round(percent * bar_length)-1) + '>'
        spaces = ' ' * (bar_length - len(arrow))
        sys.stdout.write("\rPercent: [{0}] {1}%".format(arrow + spaces, int(round(percent * 100))))
        sys.stdout.flush()



"""This is used to check the os requirements"""
from os import path
from shutil import which
from libpkpass.commands.command import Command

    ####################################################################
class VerifyInstall(Command):
    """This class is used as a command object and parses information passed through
    the CLI to show passwords that have been distributed to users"""
    ####################################################################
    name = 'verifyinstall'
    description = 'verify required software is present'
    selected_args = Command.selected_args + ['pwstore', 'keypath', 'noverify', 'card_slot',
                                             'connect']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                  """
        ####################################################################
        print_messages(check_required_software, "installed software check")
        print_messages(check_passdb, "passdb check",
                       cabundle=path.realpath(self.args['cabundle']),
                       pwstore=path.realpath(self.args['pwstore']),
                       keypath=path.realpath(self.args['keypath']),
                       certpath=path.realpath(self.args['certpath']))

        ####################################################################
    def _validate_args(self):
        ####################################################################
        pass

    ####################################################################
def check_exists(name):
    """ Check whether a program exists in path"""
    ####################################################################
    return which(name) is not None

    ####################################################################
def print_messages(func, msg, **kwargs):
    ####################################################################
    print("Starting %s" % msg)
    print(func(**kwargs))

    ####################################################################
def check_required_software():
    ####################################################################
    required_tools = {
        'pkcs15-tool (available via opensc)': ['pkcs15-tool'],
        'ssl (openssl or libressl)': ['openssl', 'libressl']
    }
    not_found = []
    for key, value in required_tools.items():
        found_tool = False
        for tool in value:
            if check_exists(tool):
                found_tool = True
        if not found_tool:
            not_found.append(key)
    if not_found:
        return "The following packages were not found: \n\t%s" % "\n\t".join(not_found)
    return "Successful installed software check"

    ####################################################################
def check_passdb(cabundle, pwstore, certpath=None, keypath=None):
    ####################################################################
    ret_msg = []
    if not path.isfile(cabundle):
        ret_msg.append("\tCabundle is not a file")
    if not path.isdir(pwstore):
        ret_msg.append("pwstore is not a directory")
    if not path.isdir(certpath) and not path.isdir(keypath):
        ret_msg.append("Neither certpath nor keypath are directories")
    return "\n\t".join(ret_msg)



"""This Module allows for copying a password directly to the clipboard"""
import time
import os
import pyperclip
from libpkpass.commands.command import Command
from libpkpass.password import PasswordEntry
from libpkpass.errors import CliArgumentError


    ####################################################################
class Clip(Command):
    """This class allows for the copying of a password to the clipboard"""
    ####################################################################
    name = 'clip'
    description = 'Copy a password to clipboard'
    selected_args = Command.selected_args + ['pwname', 'pwstore', 'stdin', 'time', 'keypath',
                                             'nopassphrase', 'noverify', 'card_slot']

        ####################################################################
    def _run_command_execution(self):
        """ Run function for class.                                      """
        ####################################################################
        password = PasswordEntry()
        password.read_password_data(os.path.join(
            self.args['pwstore'], self.args['pwname']))
        myidentity = self.identities.iddb[self.args['identity']]

        plaintext_pw = password.decrypt_entry(
            identity=myidentity,
            passphrase=self.passphrase,
            card_slot=self.args["card_slot"])

        if not self.args['noverify']:
            result = password.verify_entry(
                myidentity['uid'], self.identities)
            if not result['sigOK']:
                print("Warning:  Could not verify that '%s' correctly signed your password entry." %
                      result['distributor'])
            if not result['certOK']:
                print("Warning:  Could not verify the certificate authenticity of user '%s'." %
                      result['distributor'])

        oldclip = pyperclip.paste()
        try:
            pyperclip.copy(plaintext_pw)
            print("Password copied into paste buffer for %s seconds" %
                  self.args['time'])
            time.sleep(self.args['time'])
        finally:
            pyperclip.copy(oldclip)

        ####################################################################
    def _validate_args(self):
        ####################################################################
        for argument in ['keypath']:
            if argument not in self.args or self.args[argument] is None:
                raise CliArgumentError(
                    "'%s' is a required argument" % argument)



#!/usr/bin/env python
"""This module tests the show module"""
import unittest
import argparse
import mock
import libpkpass.commands.cli as cli
import libpkpass.commands.show as show
from libpkpass.errors import DecryptionError, CliArgumentError
from .basetest.basetest import CONFIG, BADPIN

class ShowErrors(unittest.TestCase):
    """This class tests the show class"""

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='show', identity='r1',
                                                nopassphrase="true",
                                                all=None,
                                                pwname='test',
                                                config=CONFIG))
    def test_decryption(self, subparser_name):
        """test decryption functionality"""
        ret = True
        try:
            show.Show(cli.Cli())
        except DecryptionError as error:
            if error.msg == BADPIN:
                ret = False
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='show', identity='bleh',
                                                nopassphrase="true",
                                                all=None,
                                                pwname='test',
                                                config=CONFIG))
    def test_recipient_not_in_database(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            show.Show(cli.Cli())
        except CliArgumentError as error:
            if error.msg == "Error: Your user 'bleh' is not in the recipient database":
                ret = True
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='show', identity='r1',
                                                nopassphrase="true",
                                                all=True,
                                                pwname='*test*',
                                                config=CONFIG))
    def test_showall_decryption(self, subparser_name):
        """test decryption functionality"""
        ret = True
        try:
            show.Show(cli.Cli())
        except DecryptionError as error:
            if error.msg == BADPIN:
                ret = False
        self.assertTrue(ret)

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(subparser_name='show', identity='r1',
                                                nopassphrase="true",
                                                all=None,
                                                config=CONFIG))
    def test_show_nopass_error(self, subparser_name):
        """test decryption functionality"""
        ret = False
        try:
            show.Show(cli.Cli())
        except KeyError as error:
            if str(error) == "'pwname'":
                ret = str(error)
        self.assertEqual(ret, "'pwname'")


if __name__ == '__main__':
    unittest.main()
